package com.example.ellegadodepintia.repositorios

import com.example.ellegadodepintia.PlayerLeaderBoard
import com.example.ellegadodepintia.exploradoresDePintia.model.GameState
import com.google.firebase.firestore.FirebaseFirestore
import com.google.firebase.firestore.SetOptions

object RepositorioJugador {
    private val db by lazy { FirebaseFirestore.getInstance() }
    private lateinit var email: String

    fun cargarJugador(email: String) {
        RepositorioJugador.email = email
        db.collection("users").document(email).get().addOnSuccessListener {
            GameState.jugador.monedas = (it.get("monedas") as Long).toInt()
            GameState.jugador.nivelCiudad = (it.get("nivelCiudad") as Long).toInt()

            val objetosTienda = (it.get("objetosTienda") as? List<*>)
                ?.filterIsInstance<String>()
                ?.toCollection(ArrayList())
                ?: ArrayList()

            GameState.jugador.objetosTienda = objetosTienda

            val objetosArmeria = (it.get("objetosArmeria") as? List<*>)
                ?.filterIsInstance<String>()
                ?.toCollection(ArrayList())
                ?: ArrayList()

            GameState.jugador.objetosArmeria = objetosArmeria

            val inventario = (it.get("inventario") as? List<*>)
                ?.filterIsInstance<String>()
                ?.toCollection(ArrayList())
                ?: ArrayList()

            for(nombreObjeto in inventario){
                GameState.jugador.inventario.add(RepositorioObjetos.obtenerObjetoPorNombre(nombreObjeto)!!)
            }
        }
    }

    fun addJugador(email: String, username: String) {
        db.collection("users").document(email).set(
            hashMapOf(
                "monedas" to 0,
                "username" to username,
                "nivelCiudad" to 1,
                "inventario" to ArrayList<String>(),
                "objetosTienda" to RepositorioObjetos.obtenerObjetosAleatoriosTienda(2),
                "objetosArmeria" to RepositorioObjetos.obtenerObjetosAleatoriosArmeria(2)
            )
        )
    }

    fun setMonedas(monedas: Int) {
        val userRef = db.collection("users").document(email)
        userRef.set(hashMapOf("monedas" to monedas), SetOptions.merge())
    }

    fun setObjetosTienda(objetosTienda : ArrayList<String>) {
        val userRef = db.collection("users").document(email)
        userRef.set(hashMapOf("objetosTienda" to objetosTienda), SetOptions.merge())
    }

    fun setObjetosArmeria(objetosArmeria : ArrayList<String>) {
        val userRef = db.collection("users").document(email)
        userRef.set(hashMapOf("objetosArmeria" to objetosArmeria), SetOptions.merge())
    }

    fun setNivelTaberna(nivelCiudad : Int) {
        val userRef = db.collection("users").document(email)
        userRef.set(hashMapOf("nivelCiudad" to nivelCiudad), SetOptions.merge())
    }

    fun setInventario(objetos : ArrayList<String>) {
        val userRef = db.collection("users").document(email)
        userRef.set(hashMapOf("inventario" to objetos), SetOptions.merge())
    }

    fun cargarJugadores(callback: (List<PlayerLeaderBoard>) -> Unit) {
        db.collection("users")
            .get()
            .addOnSuccessListener { result ->
                val players = mutableListOf<PlayerLeaderBoard>()

                for (document in result) {
                    val name = document.getString("username").toString()
                    val score = document.getLong("monedas")?.toInt() ?: 0
                    players.add(
                        PlayerLeaderBoard(
                            0,
                            name,
                            score
                        )
                    )
                }
                players.sortByDescending { it.score }
                players.forEachIndexed { index, player ->
                    player.position = index + 1
                }

                callback(players)
            }
    }

}
